@@ -510,23 +510,35 @@ soup_cookie_jar_add_cookie (SoupCookieJar *jar, SoupCookie *cookie)
 	soup_cookie_jar_changed (jar, NULL, cookie);
 }
 
+static const char *
+normalize_cookie_domain (const char *domain)
+{
+	/* Trim any leading dot if present to transform the cookie
+         * domain into a valid hostname.
+         */
+	if (domain != NULL && domain[0] == '.')
+		return domain + 1;
+	return domain;
+}
+
 static gboolean
 incoming_cookie_is_third_party (SoupCookie *cookie, SoupURI *first_party)
 {
+	const char *normalized_cookie_domain;
 	const char *cookie_base_domain;
 	const char *first_party_base_domain;
 
 	if (first_party == NULL || first_party->host == NULL)
 		return TRUE;
 
-	cookie_base_domain = soup_tld_get_base_domain (cookie->domain, NULL);
+	normalized_cookie_domain = normalize_cookie_domain (cookie->domain);
+	cookie_base_domain = soup_tld_get_base_domain (normalized_cookie_domain, NULL);
 	if (cookie_base_domain == NULL)
 		cookie_base_domain = cookie->domain;
 
 	first_party_base_domain = soup_tld_get_base_domain (first_party->host, NULL);
 	if (first_party_base_domain == NULL)
 		first_party_base_domain = first_party->host;
-
 	return !soup_host_matches_host (cookie_base_domain, first_party_base_domain);
 }
 
